@@ -7,11 +7,11 @@
 ## http://search.cpan.org/dist/Devel-NYTProf/
 ##
 ###########################################################
-## $Id: NYTProf.pm 719 2009-03-24 09:21:49Z tim.bunce $
+## $Id: NYTProf.pm 780 2009-06-23 20:33:57Z tim.bunce $
 ###########################################################
 package Devel::NYTProf;
 
-our $VERSION = '2.09';
+our $VERSION = '2.10';
 
 package    # hide the package from the PAUSE indexer
     DB;
@@ -261,6 +261,9 @@ example:
 
   export NYTPROF=trace=2:start=init:file=/tmp/nytprof.out
 
+Any colon or equal characters in a value can be escaped by preceeding them with
+a backslash.
+
 =head2 addpid=1
 
 Append the current process id to the end of the filename.
@@ -421,17 +424,30 @@ collecting profile data.
 Using the C<start=no> option lets you leave the profiler disabled initially
 until you call DB::enable_profile() at the right moment.
 
-You can finish profiling completely by calling DB::finish_profile().
-This may be useful if perl is exiting abnormally, leaving the profile data file
-in an incomplete state,
+The profile output file can't be used until it's been properly completed and
+closed.  Calling DB::disable_profile() doesn't do that.  To make a profile file
+usable before the profiled application has completed you can call
+DB::finish_profile(). Alternatively you could call DB::enable_profile($newfile).
 
-=head2 Multiple Output Files
+=head2 DB::disable_profile()
 
-You can pass a filename argument to DB::enable_profile() to make NYTProf write
-future profile data to that file. The current output file, if any, is closed.
-Any existing file with the new name will be deleted before being written to.
-When combined with DB::disable_profile() this lets you profile individual
-sections of code.
+Stops collection of profile data.
+
+Subroutine calls which were made while profiling was enabled and are still on
+the call stack (have not yet exited) will still have their profile data
+collected when they exit.
+
+=head2 DB::enable_profile($newfile)
+
+Enables collection of profile data. If $newfile is true the profile data will be
+written to $newfile (after completing and closing the previous file, if any).
+If $newfile already exists it will be deleted first.
+
+=head2 DB::finish_profile()
+
+Calls DB::disable_profile(), then completes the profile data file by writing
+subroutine profile data, and then closes the file. The in memory subroutine
+profile data is then discarded.
 
 =head1 REPORTS
 
@@ -595,6 +611,10 @@ but the cost is dispersed across possibly many calling locations).
 
 The C<goto &foo;> isn't recognised as a subroutine call by the subroutine profiler.
 
+=head2 Calls to XSubs which exit via an exception
+
+Calls to XSubs which exit via an exception are not recorded by subroutine profiler.
+
 =head2 #line directives
 
 The reporting code currently doesn't handle #line directives, but at least it
