@@ -8,12 +8,13 @@ use Config;
 use ExtUtils::testlib;
 use Getopt::Long;
 use Test::More;
+use Data::Dumper;
 
 use base qw(Exporter);
 our @EXPORT = qw(run_test_group);
 
 use Devel::NYTProf::Reader;
-use Devel::NYTProf::Util qw(strip_prefix_from_paths);
+use Devel::NYTProf::Util qw(strip_prefix_from_paths html_safe_filename);
 
 
 my $profile_datafile = 'nytprof_t.out';     # non-default to test override works
@@ -25,7 +26,7 @@ my %opts = (
     profperlopts => '-d:NYTProf',
     html         => $ENV{NYTPROF_TEST_HTML},
 );
-GetOptions(\%opts, qw/p=s I=s v|verbose d|debug html open profperlopts=s leave=i use_db_sub=i/)
+GetOptions(\%opts, qw/p=s I=s v|verbose d|debug html open profperlopts=s leave=i use_db_sub=i savesrc=i compress=i abort/)
     or exit 1;
 
 $opts{v}    ||= $opts{d};
@@ -42,6 +43,15 @@ if ($ENV{NYTPROF}) {                        # avoid external interference
 # options the user wants to override when running tests
 my %NYTPROF_TEST = map { split /=/, $_, 2 } split /:/, $ENV{NYTPROF_TEST} || '';
 
+# set some NYTProf options for this process in case 'extra tests' call
+# Devel::NYTProf::Data methods directly. This is a hack because the options
+# are global and there's no way to discover defaults or restore previous values.
+# So we just do trace for now.
+for my $opt (qw(trace)) {
+    DB::set_option($opt, $NYTPROF_TEST{$opt}) if defined $NYTPROF_TEST{$opt};
+}
+
+
 # but we'll force a specific test data file
 $NYTPROF_TEST{file} = $profile_datafile;
 
@@ -50,7 +60,7 @@ chdir('t') if -d 't';
 if (-d '../blib') {
     unshift @INC, '../blib/arch', '../blib/lib';
 }
-my $bindir      = (grep {-d} qw(./bin ../bin))[0];
+my $bindir      = (grep {-d} qw(./blib/script ../blib/script))[0];
 my $nytprofcsv  = "$bindir/nytprofcsv";
 my $nytprofhtml = "$bindir/nytprofhtml";
 
@@ -63,26 +73,33 @@ $perl = ".$perl" if $perl =~ m|^\./|;
 
 my @test_opt_leave      = (defined $opts{leave})      ? ($opts{leave})      : (1, 0);
 my @test_opt_use_db_sub = (defined $opts{use_db_sub}) ? ($opts{use_db_sub}) : (0, 1);
+my @test_opt_savesrc    = (defined $opts{savesrc})    ? ($opts{savesrc})    : (0, 1);
+my @test_opt_compress   = (defined $opts{compress})   ? ($opts{compress})   : (0, 1);
 
 # build @env_combinations
 my @env_combinations;
 for my $leave (@test_opt_leave) {
     for my $use_db_sub (@test_opt_use_db_sub) {
-        push @env_combinations, {
-            start      => 'init',
-            leave      => $leave,
-            use_db_sub => $use_db_sub,
+        for my $savesrc (@test_opt_savesrc) {
+            for my $compress (@test_opt_compress) {
+                push @env_combinations, {
+                    start      => 'init',
+                    leave      => $leave,
+                    use_db_sub => $use_db_sub,
+                    savesrc    => $savesrc,
+                    compress   => $compress,
+                }
+            }
         }
     }
 }
 
 
 sub run_test_group {
-    my ($test_count, $test_code) = @_;
-
-    # no warnings "undefined value";
-    $test_count ||= 0;
-    $test_count = 0 unless $test_code;
+    my ($opts) = @_;
+    my $override_env     = $opts->{override_env} || {};
+    my $extra_test_code  = $opts->{extra_test_code};
+    my $extra_test_count = $opts->{extra_test_count} || 0;
 
     # obtain group from file name
     my $group;
@@ -101,36 +118,71 @@ sub run_test_group {
         print "nytprofcvs: $nytprofcsv\n";
     }
 
-    my $tests_per_env = number_of_tests(@tests) + $test_count;
+    my $tests_per_env = number_of_tests(@tests) + $extra_test_count;
 
     plan tests => 1 + $tests_per_env * @env_combinations;
 
     # Windows emulates the executable bit based on file extension only
     ok($^O eq "MSWin32" ? -f $nytprofcsv : -x $nytprofcsv, "Found nytprofcsv as $nytprofcsv");
 
+    my %env_influence;
+
     for my $env (@env_combinations) {
+        my $prev_failures = count_of_failed_tests();
+
+        my %env = (%$env, %$override_env, %NYTPROF_TEST);
+        local $ENV{NYTPROF} = join ":", map {"$_=$env{$_}"} sort keys %env;
+        my $context = "NYTPROF=$ENV{NYTPROF}\n";
+        ($opts{v}) ? warn $context : print $context;
+
         for my $test (@tests) {
-            run_test_with_env($test, $env);
+            run_test($test);
         }
 
-        if ($test_code) {
+        if ($extra_test_code) {
+            note("running $extra_test_count extra tests...");
             my $profile = eval { Devel::NYTProf::Data->new({filename => $profile_datafile}) };
             if ($@) {
                 diag($@);
-                fail("extra tests group '$group'") foreach (1 .. $test_count);
+                fail("extra tests group '$group'") foreach (1 .. $extra_test_count);
                 return;
             }
 
-            $test_code->($profile, $env);
+            $extra_test_code->($profile, $env);
         }
+
+        # did any tests fail?
+        my $failed = (count_of_failed_tests() - $prev_failures) ? 1 : 0;
+        # record what env settings may have influenced the failure
+        ++$env_influence{$_}{$env->{$_}}{$failed ? 'fail' : 'pass'} for keys %$env;
     }
+
+    # report which env vars influenced the failures, if any
+    my @env_influence;
+    for my $envvar (sort keys %env_influence) {
+        my $variants = $env_influence{$envvar};
+        local $Data::Dumper::Indent   = 0;
+        local $Data::Dumper::Sortkeys = 1;
+        local $Data::Dumper::Terse    = 1;
+        local $Data::Dumper::Quotekeys= 0;
+        local $Data::Dumper::Pair     = ' ';
+        $variants->{$_} = Dumper($variants->{$_}) for keys %$variants;
+        my $v = (values %$variants)[0]; # use one as a reference
+        # all the same?
+        next if keys %$variants == grep { $_ eq $v } values %$variants;
+        push @env_influence, sprintf "%15s: %s\n", $envvar,
+            join ', ', map { "$_ => $variants->{$_}" } sort keys %$variants;
+    }
+    if (@env_influence) {
+        diag "Test failures of $group related to settings:";
+        diag $_ for @env_influence;
+    }
+
 }
 
-sub run_test_with_env {
-    my ($test, $env, $test_code) = @_;
 
-    my %env = (%$env, %NYTPROF_TEST);
-    local $ENV{NYTPROF} = join ":", map {"$_=$env{$_}"} sort keys %env;
+sub run_test {
+    my ($test) = @_;
 
     #print $test . '.'x (20 - length $test);
     $test =~ / (.+?) \. (?:(\d)\.)? (\w+) $/x or do {
@@ -149,7 +201,7 @@ sub run_test_with_env {
         verify_data($test, $test_datafile);
     }
     elsif ($type eq 'x') {
-        my $outdir = "$basename.outdir";
+        my $outdir = $basename.'_outdir';
         mkdir $outdir or die "mkdir($outdir): $!" unless -d $outdir;
         unlink <$outdir/*>;
 
@@ -167,10 +219,17 @@ sub run_test_with_env {
     else {
         warn "Unrecognized extension '$type' on test file '$test'\n";
     }
+
+    if ($opts{abort}) {
+        my $test_builder = Test::More->builder;
+        my @summary = $test_builder->summary;
+        BAIL_OUT("Aborting after test failure")
+            if grep { !$_ } @summary;
+    }
 }
 
 sub run_command {
-    my ($cmd) = @_;
+    my ($cmd, $show_stdout) = @_;
     warn "NYTPROF=$ENV{NYTPROF}\n" if $opts{v} && $ENV{NYTPROF};
     local $ENV{PERL5LIB} = $perl5lib;
     warn "$cmd\n" if $opts{v};
@@ -178,7 +237,13 @@ sub run_command {
     open(RV, "$cmd |") or die "Can't execute $cmd: $!\n";
     my @results = <RV>;
     my $ok = close RV;
-    warn "Error status $? from $cmd!\n\n" if not $ok;
+    if (not $ok) {
+        warn "Error status $? from $cmd!\n";
+        warn "NYTPROF=$ENV{NYTPROF}\n" if $ENV{NYTPROF} and not $opts{v};
+        $show_stdout = 1;
+        sleep 2;
+    }
+    if ($show_stdout) { warn $_ for @results }
     return $ok;
 }
 
@@ -201,14 +266,16 @@ sub verify_data {
         return;
     }
 
-    $profile->normalize_variables;
-    dump_profile_to_file($profile, "$test.new");
-    my @got      = slurp_file("$test.new");
-    my @expected = slurp_file($test);
-
-    is_deeply(\@got, \@expected, "$test match generated profile data")
-        ? unlink("$test.new")
-        : diff_files($test, "$test.new");
+    SKIP: {
+        skip 'Expected profile data does not have VMS paths',1 if(($^O eq 'VMS') && ($test =~m/test60|test14/i));
+	$profile->normalize_variables;
+        dump_profile_to_file($profile, $test.'_new');
+        my @got      = slurp_file($test.'_new');
+        my @expected = slurp_file($test);
+        is_deeply(\@got, \@expected, "$test match generated profile data")
+            ? unlink($test.'_new')
+            : diff_files($test, $test.'_new');
+    }
 }
 
 
@@ -259,6 +326,7 @@ sub verify_csv_report {
     # foo.pm => foo.pm.csv is tested by foo.pm.x
     $csvfile =~ s/\.x//;
     $csvfile .= ".p" unless $csvfile =~ /\.p/;
+    $csvfile = html_safe_filename($csvfile);
     $csvfile = "$outdir/${csvfile}-line.csv";
     unlink $csvfile;
 
@@ -285,8 +353,10 @@ sub verify_csv_report {
 
     # if it was slower than expected then we're very generous, to allow for
     # slow systems, e.g. cpan-testers running in cpu-starved virtual machines.
-    my $max_time_overrun_percentage = ($ENV{AUTOMATED_TESTING}) ? 300 : 200;
-    my $max_time_underrun_percentage = 95;
+    # e.g., http://www.nntp.perl.org/group/perl.cpan.testers/2009/06/msg4227689.html
+    my $max_time_overrun_percentage = ($ENV{AUTOMATED_TESTING}) ? 400 : 200;
+    # e.g., http://www.nntp.perl.org/group/perl.cpan.testers/2009/06/msg4230206.html
+    my $max_time_underrun_percentage = 90;
 
     my @accuracy_errors;
     $index = 0;
@@ -331,8 +401,8 @@ sub verify_csv_report {
     }
 
     is_deeply(\@got, \@expected, "$test match generated CSV data") or do {
-        spit_file("$test.new", join("", @got));
-        diff_files($test, "$test.new");
+        spit_file($test.'_new', join("", @got));
+        diff_files($test, $test.'_new');
     };
     is(join("\n", @accuracy_errors), '', "$test times should be reasonable");
 }
@@ -396,6 +466,12 @@ sub unlink_old_profile_datafiles {
 }
 
 
+sub count_of_failed_tests {
+    my @details = Test::Builder->new->details;
+    return scalar grep { not $_->{ok} } @details;
+}
+
+
 1;
 
-# vim:ts=8:sw=2
+# vim:ts=8:sw=4
